from .core import Engine
from .scene import Scene
from .entity import Entity, Player, Rectangle
from .camera import Camera
from .time import Time
from .utils import load_texture
from .ui import UIElement, Button, Label


# src/saturn2d/utils.py
import pygame
import requests
from io import BytesIO

def load_texture(path_or_url):
    """
    Load a texture from a local file or a web URL.
    Returns a pygame.Surface with alpha.
    """
    if path_or_url.startswith("http"):
        response = requests.get(path_or_url)
        image = pygame.image.load(BytesIO(response.content)).convert_alpha()
    else:
        image = pygame.image.load(path_or_url).convert_alpha()
    return image


# src/saturn2d/camera.py
class Camera:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.x = 0
        self.y = 0

    def follow(self, target):
        self.x = target.x + target.width // 2 - self.width // 2
        self.y = target.y + target.height // 2 - self.height // 2
        self.x = max(0, self.x)
        self.y = max(0, self.y)


# src/saturn2d/entity.py
import pygame

class Entity:
    def __init__(self, x=0, y=0, width=32, height=32, sprite=None):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.sprite = sprite
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)

    def handle_event(self, event):
        pass

    def update(self, dt, obstacles=None):
        self.rect.topleft = (self.x, self.y)

    def render(self, screen, camera=None):
        draw_x, draw_y = self.x, self.y
        if camera:
            draw_x -= camera.x
            draw_y -= camera.y
        if self.sprite:
            screen.blit(self.sprite, (draw_x, draw_y))
        else:
            pygame.draw.rect(screen, (255, 255, 255), (draw_x, draw_y, self.width, self.height))

    def collides_with(self, other):
        return self.rect.colliderect(other.rect)


class Player(Entity):
    def __init__(self, x, y, width=32, height=32, sprite=None, speed=200):
        super().__init__(x, y, width, height, sprite)
        self.speed = speed
        self.is_player = True

    def update(self, dt, obstacles=None):
        keys = pygame.key.get_pressed()
        dx = dy = 0
        if keys[pygame.K_LEFT] or keys[pygame.K_a]:
            dx -= self.speed * dt
        if keys[pygame.K_RIGHT] or keys[pygame.K_d]:
            dx += self.speed * dt
        if keys[pygame.K_UP] or keys[pygame.K_w]:
            dy -= self.speed * dt
        if keys[pygame.K_DOWN] or keys[pygame.K_s]:
            dy += self.speed * dt

        # Horizontal movement & collision
        self.x += dx
        self.rect.topleft = (self.x, self.y)
        if obstacles:
            for o in obstacles:
                if self.collides_with(o):
                    if dx > 0:
                        self.x = o.rect.left - self.width
                    elif dx < 0:
                        self.x = o.rect.right
                    self.rect.topleft = (self.x, self.y)

        # Vertical movement & collision
        self.y += dy
        self.rect.topleft = (self.x, self.y)
        if obstacles:
            for o in obstacles:
                if self.collides_with(o):
                    if dy > 0:
                        self.y = o.rect.top - self.height
                    elif dy < 0:
                        self.y = o.rect.bottom
                    self.rect.topleft = (self.x, self.y)


class Rectangle(Entity):
    def __init__(self, x, y, width, height, color=(255, 255, 255)):
        super().__init__(x, y, width, height)
        self.color = color

    def render(self, screen, camera=None):
        draw_x, draw_y = self.x, self.y
        if camera:
            draw_x -= camera.x
            draw_y -= camera.y
        pygame.draw.rect(screen, self.color, (draw_x, draw_y, self.width, self.height))


# src/saturn2d/ui.py
import pygame


class UIElement:
    def __init__(self, x, y, width, height):
        self.rect = pygame.Rect(x, y, width, height)
        self.visible = True

    def handle_event(self, event):
        pass

    def update(self, dt):
        pass

    def render(self, screen):
        pass


class Label(UIElement):
    def __init__(self, x, y, text, font_size=24, color=(255, 255, 255)):
        self.font = pygame.font.SysFont(None, font_size)
        self.text = text
        self.color = color
        text_surface = self.font.render(text, True, color)
        width, height = text_surface.get_size()
        super().__init__(x, y, width, height)

    def set_text(self, new_text):
        self.text = new_text

    def render(self, screen):
        if not self.visible:
            return
        text_surface = self.font.render(self.text, True, self.color)
        screen.blit(text_surface, self.rect.topleft)


class Button(UIElement):
    def __init__(self, x, y, width, height, text, onclick=None,
                 bg_color=(50, 50, 50),
                 hover_color=(80, 80, 80),
                 text_color=(255, 255, 255)):

        super().__init__(x, y, width, height)
        self.onclick = onclick
        self.bg_color = bg_color
        self.hover_color = hover_color
        self.text_color = text_color

        self.font = pygame.font.SysFont(None, 24)
        self.text = text
        self.hovered = False

    def handle_event(self, event):
        if event.type == pygame.MOUSEMOTION:
            self.hovered = self.rect.collidepoint(event.pos)

        if event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 1 and self.hovered:
                if self.onclick:
                    self.onclick()

    def render(self, screen):
        if not self.visible:
            return

        color = self.hover_color if self.hovered else self.bg_color
        pygame.draw.rect(screen, color, self.rect)

        text_surface = self.font.render(self.text, True, self.text_color)
        text_rect = text_surface.get_rect(center=self.rect.center)
        screen.blit(text_surface, text_rect)


# src/saturn2d/time.py
import time

class Time:
    def __init__(self):
        self.last_time = time.time()
        self.delta_time = 0

    def update(self):
        current = time.time()
        self.delta_time = current - self.last_time
        self.last_time = current


# src/saturn2d/core.py
import pygame
from .time import Time


class Engine:
    def __init__(self, width=800, height=600, title="Saturn2D Game", fps=60):
        pygame.init()
        self.screen = pygame.display.set_mode((width, height))
        pygame.display.set_caption(title)

        self.clock = pygame.time.Clock()
        self.fps = fps
        self.running = False

        self.current_scene = None
        self.next_scene = None

        self.time = Time()

        # Transition variables
        self.transitioning = False
        self.transition_alpha = 0
        self.transition_speed = 300  # fade speed
        self.fade_surface = pygame.Surface((width, height))
        self.fade_surface.fill((0, 0, 0))

    def set_scene(self, scene):
        self.current_scene = scene

    def change_scene(self, new_scene):
        self.next_scene = new_scene
        self.transitioning = True
        self.transition_alpha = 0

    def _handle_transition(self, dt):
        if not self.transitioning:
            return

        # Fade out
        self.transition_alpha += self.transition_speed * dt

        if self.transition_alpha >= 255:
            self.transition_alpha = 255

            # Switch scene when fully black
            if self.next_scene:
                self.current_scene = self.next_scene
                self.next_scene = None

            # Start fading back in
            self.transition_speed = -self.transition_speed

        # Fade in finished
        if self.transition_alpha <= 0:
            self.transition_alpha = 0
            self.transition_speed = abs(self.transition_speed)
            self.transitioning = False

    def run(self):
        self.running = True

        while self.running:
            self.time.update()
            dt = self.time.delta_time

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False

                if self.current_scene:
                    self.current_scene.handle_event(event)

            if self.current_scene:
                self.current_scene.update(dt)
                self.current_scene.render(self.screen)

            # Handle transition
            self._handle_transition(dt)

            # Draw fade overlay
            if self.transitioning:
                self.fade_surface.set_alpha(int(self.transition_alpha))
                self.screen.blit(self.fade_surface, (0, 0))

            pygame.display.flip()
            self.clock.tick(self.fps)

        pygame.quit()


# src/saturn2d/scene.py

class Scene:
    def __init__(self):
        self.entities = []
        self.ui_elements = []   # 👈 NEW
        self.camera = None

    def add_entity(self, entity):
        self.entities.append(entity)

    def add_ui(self, ui_element):
        self.ui_elements.append(ui_element)

    def handle_event(self, event):
        # Send events to world entities
        for entity in self.entities:
            entity.handle_event(event)

        # Send events to UI
        for ui in self.ui_elements:
            ui.handle_event(event)

    def update(self, dt):
        for entity in self.entities:
            obstacles = [e for e in self.entities if e != entity] \
                if getattr(entity, "is_player", False) else None
            entity.update(dt, obstacles=obstacles)

        for ui in self.ui_elements:
            ui.update(dt)

        if self.camera and self.entities:
            self.camera.follow(self.entities[0])

    def render(self, screen):
        screen.fill((0, 0, 0))

        # Render world
        for entity in self.entities:
            entity.render(screen, camera=self.camera)

        # Render UI (NO camera offset)
        for ui in self.ui_elements:
            ui.render(screen)